#ifndef __MACRO_H__
#define __MACRO_H__
#include <iostream>
#include <string>
#include <vector>
using namespace std;
/** @addtogroup main */
// @{ 
/// flag to select symetric seeds only
extern bool                   gv_symetric;
/// number of runs (0 when full enumeration)
extern int                    gv_nbruns;
/// flag that enables the "overdubbed" symbols
extern bool                   gv_matching_symbol_flag;

/// alignment alphabet size
extern int                    gv_align_alphabet_size;
/// seed alphabet size
extern int                    gv_seed_alphabet_size;
/// signature vector (number of seed letters per seed)
extern vector<int>            gv_signature;
/// signature flag that enable the signature selection
/// @see gv_signature
extern bool                   gv_signature_flag;
/// weight for each seed letter
extern vector<double>         gv_bsel_weight;

/// min seed letter selectivity
extern double                 gv_bsel_minprob;
/// max seed letter selectivity
extern double                 gv_bsel_maxprob;

/// min seed span
extern int                    gv_minspan;
/// max seed span
extern int                    gv_maxspan;

/// min seed weight
extern double                 gv_minweight;
/// max seed weight
extern double                 gv_maxweight;

/// flag that enables min and max seed weight
/// @see gv_minweight,gv_maxweight
extern bool                   gv_weight_interval;
/// flag that enables vectorized seeds
/// @see gv_vectorizedsubsetseed_scoring_matrix,gv_vectorizedsubsetseed_scoring_threshold
extern bool                   gv_vectorized_flag;
/// set of symbols when "overdubbed"
/// @see gv_bsymbols_flag
extern char *                 gv_bsymbols_array;
/// flag that enables "overdubbed" symbols
/// @see gv_bsymbols_array
extern bool                   gv_bsymbols_flag;
// @}
 
/*
 * (I) Usefull functions
 */

#define ASSERT(ptr)                                                   \
  if (!(ptr)){                                                        \
    cerr << "*ERROR : memory allocation failed on" << #ptr <<   endl; \
    exit(-1);                                                         \
  }

#define ERROR(message1,message2) {                              \
  cerr << "*ERROR : " << message1 << " : " << message2 << endl; \
  exit(-1);                                                     \
}

#define WARNING(message1,message2) {                              \
  cerr << "*WARNING : " << message1 << " : " << message2 << endl; \
}

#define OUTSTREAM (gv_ofstream.is_open()?gv_ofstream:cout)
#define MAX(a,b) ((a)>(b)?(a):(b)) 
#define MAX3(a,b,c) MAX((a),MAX((b),(c)))
#define MIN(a,b) ((a)<(b)?(a):(b))
#define MIN3(a,b,c) MIN((a),MIN((b),(c)))
#define ABS(a) ((a)>=0?(a):(-a))

#define INT_INFINITY   (0x7fffffff)  

#define BINARY(u,r,o)                             \
  for (int _i__ = 1; _i__ <  (1<<r) ;_i__ <<=1){  \
     if (_i__&u)                                  \
        o << "1";                                 \
     else                                         \
        o << "0";                                 \
  }

int power(int x, int n);
double power(double x, int n);

/*
 *  (II) Constants
 */

#define FALSE      0
#define TRUE       1
#define KILO      (1024)
#define MEGA      (KILO*KILO)
#define GIGA      (MEGA*KILO)


/*
 *  (III) specific definitions 
 */

#define XSET         (unsigned int)
#define XSETBITSIZE  (sizeof(XSET)*8)

//#define ASSERTB
#define USEQUEUE
//#define BUILD
#define USEMAP


#endif  

